import unittest

from NGram.AdditiveSmoothing import AdditiveSmoothing
from test.SimpleSmoothingTest import SimpleSmoothingTest


class AdditiveSmoothingTest(SimpleSmoothingTest, unittest.TestCase):
    delta1: float
    delta2: float
    delta3: float

    def setUp(self) -> None:
        super().setUp()
        additiveSmoothing = AdditiveSmoothing()
        self.complexUniGram.calculateNGramProbabilitiesTrained(self.validationCorpus, additiveSmoothing)
        self.delta1 = additiveSmoothing.getDelta()
        self.complexBiGram.calculateNGramProbabilitiesTrained(self.validationCorpus, additiveSmoothing)
        self.delta2 = additiveSmoothing.getDelta()
        self.complexTriGram.calculateNGramProbabilitiesTrained(self.validationCorpus, additiveSmoothing)
        self.delta3 = additiveSmoothing.getDelta()

    def test_PerplexityComplex(self):
        self.assertAlmostEqual(4043.947022, self.complexUniGram.getPerplexity(self.testCorpus), 4)
        self.assertAlmostEqual(9220.218871, self.complexBiGram.getPerplexity(self.testCorpus), 4)
        self.assertAlmostEqual(30695.701941, self.complexTriGram.getPerplexity(self.testCorpus), 4)

    def test_CalculateNGramProbabilitiesComplex(self):
        self.assertEqual((20000 + self.delta1) / (376019.0 + self.delta1 * (self.complexUniGram.vocabularySize() + 1)), self.complexUniGram.getProbability("<s>"))
        self.assertEqual((50 + self.delta1) / (376019.0 + self.delta1 * (self.complexUniGram.vocabularySize() + 1)), self.complexUniGram.getProbability("atatürk"))
        self.assertEqual((11 + self.delta2) / (20000.0 + self.delta2 * (self.complexBiGram.vocabularySize() + 1)), self.complexBiGram.getProbability("<s>", "mustafa"))
        self.assertEqual((3 + self.delta2) / (138.0 + self.delta2 * (self.complexBiGram.vocabularySize() + 1)), self.complexBiGram.getProbability("mustafa", "kemal"))
        self.assertEqual((1 + self.delta3) / (11.0 + self.delta3 * (self.complexTriGram.vocabularySize() + 1)), self.complexTriGram.getProbability("<s>", "mustafa", "kemal"))
        self.assertEqual((1 + self.delta3) / (3.0 + self.delta3 * (self.complexTriGram.vocabularySize() + 1)), self.complexTriGram.getProbability("mustafa", "kemal", "atatürk"))


if __name__ == '__main__':
    unittest.main()



import unittest

from NGram.NGram import NGram
from test.CorpusTest import CorpusTest


class NGramTest(CorpusTest, unittest.TestCase):

    simpleUniGram: NGram
    simpleBiGram: NGram
    simpleTriGram: NGram
    complexUniGram: NGram
    complexBiGram: NGram
    complexTriGram: NGram
    simpleCorpus : list
    trainCorpus: list
    testCorpus: list
    validationCorpus: list

    def setUp(self) -> None:
        self.simpleCorpus = [["<s>", "ali", "topu", "at", "mehmet", "ayşeye", "gitti", "</s>"],
                             ["<s>", "ali", "top", "at", "ayşe", "eve", "gitti", "</s>"],
                             ["<s>", "ayşe", "kitabı", "ver", "</s>"],
                             ["<s>", "ali", "topu", "mehmete", "at", "</s>"],
                             ["<s>", "ali", "topu", "at", "mehmet", "ayşeyle", "gitti", "</s>"]]
        self.simpleUniGram = NGram(1, self.simpleCorpus)
        self.simpleBiGram = NGram(2, self.simpleCorpus)
        self.simpleTriGram = NGram(3, self.simpleCorpus)
        self.trainCorpus = self.readCorpus("../train.txt")
        self.complexUniGram = NGram(1, self.trainCorpus)
        self.complexBiGram = NGram(2, self.trainCorpus)
        self.complexTriGram = NGram(3, self.trainCorpus)
        self.testCorpus = self.readCorpus("../test.txt")
        self.validationCorpus = self.readCorpus("../validation.txt")

    def test_GetCountSimple(self):
        self.assertEqual(5, self.simpleUniGram.getCount(["<s>"]))
        self.assertEqual(0, self.simpleUniGram.getCount(["mahmut"]), 0.0)
        self.assertEqual(1, self.simpleUniGram.getCount(["kitabı"]), 0.0)
        self.assertEqual(4, self.simpleBiGram.getCount(["<s>", "ali"]), 0.0)
        self.assertEqual(0, self.simpleBiGram.getCount(["ayşe", "ali"]), 0.0)
        self.assertEqual(0, self.simpleBiGram.getCount(["mahmut", "ali"]), 0.0)
        self.assertEqual(2, self.simpleBiGram.getCount(["at", "mehmet"]), 0.0)
        self.assertEqual(1, self.simpleTriGram.getCount(["<s>", "ali", "top"]), 0.0)
        self.assertEqual(0, self.simpleTriGram.getCount(["ayşe", "kitabı", "at"]), 0.0)
        self.assertEqual(0, self.simpleTriGram.getCount(["ayşe", "topu", "at"]), 0.0)
        self.assertEqual(0, self.simpleTriGram.getCount(["mahmut", "evde", "kal"]), 0.0)
        self.assertEqual(2, self.simpleTriGram.getCount(["ali", "topu", "at"]), 0.0)

    def test_GetCountComplex(self):
        self.assertEqual(20000, self.complexUniGram.getCount(["<s>"]), 0.0)
        self.assertEqual(50, self.complexUniGram.getCount(["atatürk"]), 0.0)
        self.assertEqual(11, self.complexBiGram.getCount(["<s>", "mustafa"]), 0.0)
        self.assertEqual(3, self.complexBiGram.getCount(["mustafa", "kemal"]), 0.0)
        self.assertEqual(1, self.complexTriGram.getCount(["<s>", "mustafa", "kemal"]), 0.0)
        self.assertEqual(1, self.complexTriGram.getCount(["mustafa", "kemal", "atatürk"]), 0.0)

    def test_VocabularySizeSimple(self):
        self.assertEqual(15, self.simpleUniGram.vocabularySize())

    def test_VocabularySizeComplex(self):
        self.assertEqual(57625, self.complexUniGram.vocabularySize(), 0.0)
        self.complexUniGram = NGram(1, self.testCorpus)
        self.assertEqual(55485, self.complexUniGram.vocabularySize(), 0.0)
        self.complexUniGram = NGram(1, self.validationCorpus)
        self.assertEqual(35663, self.complexUniGram.vocabularySize(), 0.0)

    def test_SaveAsText(self):
        self.simpleUniGram.saveAsText("simple1.txt")
        self.simpleBiGram.saveAsText("simple2.txt")
        self.simpleTriGram.saveAsText("simple3.txt")

    def test_LoadMultiPart(self):
        self.simpleUniGram = NGram(1)
        self.simpleUniGram.initWithMultipleFile("simple1part1.txt", "simple1part2.txt")
        self.simpleBiGram = NGram(2)
        self.simpleBiGram.initWithMultipleFile("simple2part1.txt", "simple2part2.txt", "simple2part3.txt")
        self.simpleTriGram = NGram(3)
        self.simpleTriGram.initWithMultipleFile("simple3part1.txt", "simple3part2.txt", "simple3part3.txt", "simple3part4.txt")
        self.test_GetCountSimple()
        self.test_VocabularySizeSimple()


if __name__ == '__main__':
    unittest.main()



from Sampling.KFoldCrossValidation import KFoldCrossValidation

from NGram.GoodTuringSmoothing import GoodTuringSmoothing
from NGram.NGram import NGram
from NGram.SimpleSmoothing import SimpleSmoothing
from NGram.TrainedSmoothing import TrainedSmoothing
import math


class InterpolatedSmoothing(TrainedSmoothing):

    __lambda1: float
    __lambda2: float
    __simpleSmoothing: SimpleSmoothing

    def __init__(self, simpleSmoothing=None):
        """
        Constructor of InterpolatedSmoothing

        PARAMETERS
        ----------
        simpleSmoothing : SimpleSmoothing
            smoothing method.
        """
        if simpleSmoothing is None:
            self.__simpleSmoothing = GoodTuringSmoothing()
        else:
            self.__simpleSmoothing = simpleSmoothing

    def __learnBestLambda(self, nGrams: list, kFoldCrossValidation: KFoldCrossValidation, lowerBound: float) -> float:
        """
        The algorithm tries to optimize the best lambda for a given corpus. The algorithm uses perplexity on the
        validation set as the optimization criterion.

        PARAMETERS
        ----------
        nGrams : list
            10 N-Grams learned for different folds of the corpus. nGrams[i] is the N-Gram trained with i'th train fold
            of the corpus.
        kFoldCrossValidation : KFoldCrossvalidation
            Cross-validation data used in training and testing the N-grams.
        lowerBound : float
            Initial lower bound for optimizing the best lambda.

        RETURNS
        -------
        float
            Best lambda optimized with k-fold crossvalidation.
        """
        bestPrevious = -1
        upperBound = 0.999
        bestLambda = (lowerBound + upperBound) / 2
        numberOfParts = 5
        testFolds = []
        for i in range(10):
            testFolds.append(kFoldCrossValidation.getTestFold(i))
        while True:
            bestPerplexity = 1000000000
            value = lowerBound
            while value <= upperBound:
                perplexity = 0
                for i in range(10):
                    nGrams[i].setLambda2(value)
                    perplexity += nGrams[i].getPerplexity(testFolds[i])
                if perplexity < bestPerplexity:
                    bestPerplexity = perplexity
                    bestLambda = value
                value += (upperBound - lowerBound) / numberOfParts
            lowerBound = self.newLowerBound(bestLambda, lowerBound, upperBound, numberOfParts)
            upperBound = self.newUpperBound(bestLambda, lowerBound, upperBound, numberOfParts)
            if bestPrevious != -1:
                if math.fabs(bestPrevious - bestPerplexity) / bestPerplexity < 0.001:
                    break
            bestPrevious = bestPerplexity
        return bestLambda

    def __learnBestLambdas(self, nGrams: list, kFoldCrossValidation: KFoldCrossValidation, lowerBound1: float,
                           lowerBound2: float) -> tuple:
        """
        The algorithm tries to optimize the best lambdas (lambda1, lambda2) for a given corpus. The algorithm uses
        perplexity on the validation set as the optimization criterion.

        PARAMETERS
        ----------
        nGrams : list
            10 N-Grams learned for different folds of the corpus. nGrams[i] is the N-Gram trained with i'th train fold
            of the corpus.
        kFoldCrossValidation : KFoldCrossValidation
            Cross-validation data used in training and testing the N-grams.
        lowerBound1 : float
            Initial lower bound for optimizing the best lambda1.
        lowerBound2 : float
            Initial lower bound for optimizing the best lambda2.

        RETURNS
        -------
        tuple
            bestLambda1 and bestLambda2
        """
        upperBound1 = 0.999
        upperBound2 = 0.999
        bestPrevious = -1
        bestLambda1 = (lowerBound1 + upperBound1) / 2
        bestLambda2 = (lowerBound2 + upperBound2) / 2
        numberOfParts = 5
        testFolds = []
        for i in range(10):
            testFolds.append(kFoldCrossValidation.getTestFold(i))
        while True:
            bestPerplexity = 1000000000
            value1 = lowerBound1
            while value1 <= upperBound1:
                value2 = lowerBound2
                while value2 <= upperBound2 and value1 + value2 < 1:
                    perplexity = 0
                    for i in range(10):
                        nGrams[i].setLambda3(value1, value2)
                        perplexity += nGrams[i].getPerplexity(testFolds[i])
                    if perplexity < bestPerplexity:
                        bestPerplexity = perplexity
                        bestLambda1 = value1
                        bestLambda2 = value2
                    value2 += (upperBound1 - lowerBound1) / numberOfParts
                value1 += (upperBound1 - lowerBound1) / numberOfParts
            lowerBound1 = self.newLowerBound(bestLambda1, lowerBound1, upperBound1, numberOfParts)
            upperBound1 = self.newUpperBound(bestLambda1, lowerBound1, upperBound1, numberOfParts)
            lowerBound2 = self.newLowerBound(bestLambda2, lowerBound2, upperBound2, numberOfParts)
            upperBound2 = self.newUpperBound(bestLambda2, lowerBound2, upperBound2, numberOfParts)
            if bestPrevious != -1:
                if math.fabs(bestPrevious - bestPerplexity) / bestPerplexity < 0.001:
                    break
            bestPrevious = bestPerplexity
        return bestLambda1, bestLambda2

    def learnParameters(self, corpus: list, N: int):
        """
        Wrapper function to learn the parameters (lambda1 and lambda2) in interpolated smoothing. The function first
        creates K NGrams with the train folds of the corpus. Then optimizes lambdas with respect to the test folds of
        the corpus depending on given N.

        PARAMETERS
        ----------
        corpus : list
            Train corpus used to optimize lambda parameters
        N : int
            N in N-Gram.
        """
        if N <= 1:
            return
        K = 10
        nGrams = []
        kFoldCrossValidation = KFoldCrossValidation(corpus, K, 0)
        for i in range(K):
            nGrams.append(NGram(N, kFoldCrossValidation.getTrainFold(i)))
            for j in range(2, N + 1):
                nGrams[i].calculateNGramProbabilitiesSimpleLevel(self.__simpleSmoothing, j)
            nGrams[i].calculateNGramProbabilitiesSimpleLevel(self.__simpleSmoothing, 1)
        if N == 2:
            self.__lambda1 = self.__learnBestLambda(nGrams, kFoldCrossValidation, 0.1)
        elif N == 3:
            (self.__lambda1, self.__lambda2) = self.__learnBestLambdas(nGrams, kFoldCrossValidation, 0.1, 0.1)

    def setProbabilities(self, nGram: NGram, level: int):
        """
        Wrapper function to set the N-gram probabilities with interpolated smoothing.

        PARAMETERS
        ----------
        nGram : NGram
            N-Gram for which the probabilities will be set.
        level : int
            Level for which N-Gram probabilities will be set. Probabilities for different levels of the N-gram can be
            set with this function. If level = 1, N-Gram is treated as UniGram, if level = 2, N-Gram is treated as
            Bigram, etc.
        """
        for j in range(2, nGram.getN() + 1):
            nGram.calculateNGramProbabilitiesSimpleLevel(self.__simpleSmoothing, j)
        nGram.calculateNGramProbabilitiesSimpleLevel(self.__simpleSmoothing, 1)
        if nGram.getN() == 2:
            nGram.setLambda2(self.__lambda1)
        elif nGram.getN() == 3:
            nGram.setLambda3(self.__lambda1, self.__lambda2)



import unittest

from NGram.NoSmoothing import NoSmoothing
from test.SimpleSmoothingTest import SimpleSmoothingTest


class NoSmoothingTest(SimpleSmoothingTest, unittest.TestCase):

    def setUp(self) -> None:
        super().setUp()
        simpleSmoothing = NoSmoothing()
        self.simpleUniGram.calculateNGramProbabilitiesSimple(simpleSmoothing)
        self.simpleBiGram.calculateNGramProbabilitiesSimple(simpleSmoothing)
        self.simpleTriGram.calculateNGramProbabilitiesSimple(simpleSmoothing)
        self.complexUniGram.calculateNGramProbabilitiesSimple(simpleSmoothing)
        self.complexBiGram.calculateNGramProbabilitiesSimple(simpleSmoothing)
        self.complexTriGram.calculateNGramProbabilitiesSimple(simpleSmoothing)

    def test_PerplexitySimple(self):
        self.assertAlmostEqual(12.318362, self.simpleUniGram.getPerplexity(self.simpleCorpus), 4)
        self.assertAlmostEqual(1.573148, self.simpleBiGram.getPerplexity(self.simpleCorpus), 4)
        self.assertAlmostEqual(1.248330, self.simpleTriGram.getPerplexity(self.simpleCorpus), 4)

    def test_PerplexityComplex(self):
        self.assertAlmostEqual(3220.299369, self.complexUniGram.getPerplexity(self.trainCorpus), 4)
        self.assertAlmostEqual(32.362912, self.complexBiGram.getPerplexity(self.trainCorpus), 4)
        self.assertAlmostEqual(2.025259, self.complexTriGram.getPerplexity(self.trainCorpus), 4)

    def test_CalculateNGramProbabilitiesSimple(self):
        self.assertEqual(5 / 35.0, self.simpleUniGram.getProbability("<s>"))
        self.assertEqual(0.0, self.simpleUniGram.getProbability("mahmut"))
        self.assertEqual(1.0 / 35.0, self.simpleUniGram.getProbability("kitabı"))
        self.assertEqual(4 / 5.0, self.simpleBiGram.getProbability("<s>", "ali"))
        self.assertEqual(0 / 2.0, self.simpleBiGram.getProbability("ayşe", "ali"))
        self.assertEqual(0.0, self.simpleBiGram.getProbability("mahmut", "ali"))
        self.assertEqual(2 / 4.0, self.simpleBiGram.getProbability("at", "mehmet"))
        self.assertEqual(1 / 4.0, self.simpleTriGram.getProbability("<s>", "ali", "top"))
        self.assertEqual(0 / 1.0, self.simpleTriGram.getProbability("ayşe", "kitabı", "at"))
        self.assertEqual(0.0, self.simpleTriGram.getProbability("ayşe", "topu", "at"))
        self.assertEqual(0.0, self.simpleTriGram.getProbability("mahmut", "evde", "kal"))
        self.assertEqual(2 / 3.0, self.simpleTriGram.getProbability("ali", "topu", "at"))

    def test_CalculateNGramProbabilitiesComplex(self):
        self.assertEqual(20000 / 376019.0, self.complexUniGram.getProbability("<s>"))
        self.assertEqual(50 / 376019.0, self.complexUniGram.getProbability("atatürk"))
        self.assertEqual(11 / 20000.0, self.complexBiGram.getProbability("<s>", "mustafa"))
        self.assertEqual(3 / 138.0, self.complexBiGram.getProbability("mustafa", "kemal"))
        self.assertEqual(1 / 11.0, self.complexTriGram.getProbability("<s>", "mustafa", "kemal"))
        self.assertEqual(1 / 3.0, self.complexTriGram.getProbability("mustafa", "kemal", "atatürk"))


if __name__ == '__main__':
    unittest.main()



from abc import abstractmethod


class SimpleSmoothing:

    @abstractmethod
    def setProbabilities(self, nGram, level: int):
        pass

    def setProbabilitiesGeneral(self, nGram):
        self.setProbabilities(nGram, nGram.getN())



class MultipleFile:

    index: int
    fileNameList: list
    lines: list
    lineIndex: int

    def __init__(self, fileList: list):
        self.fileNameList = fileList
        file = open(self.fileNameList[0])
        self.lines = file.readlines()
        self.lineIndex = 0
        self.index = 0

    def readLine(self) -> str:
        if self.lineIndex == len(self.lines):
            self.index = self.index + 1
            file = open(self.fileNameList[self.index])
            self.lines = file.readlines()
            self.lineIndex = 0
        line = self.lines[self.lineIndex]
        self.lineIndex = self.lineIndex + 1
        return line



from NGram.NGram import NGram
from NGram.NoSmoothing import NoSmoothing


class NoSmoothingWithDictionary(NoSmoothing):

    __dictionary: set

    def __init__(self, dictionary: set):
        """
        Constructor of {@link NoSmoothingWithDictionary}

        PARAMETERS
        ----------
        dictionary : set
            Dictionary to use in smoothing
        """
        self.__dictionary = dictionary

    def setProbabilities(self, nGram: NGram, level: int):
        """
        Wrapper function to set the N-gram probabilities with no smoothing and replacing unknown words not found in
        set the dictionary.

        PARAMETERS
        ----------
        nGram : NGram
            N-Gram for which the probabilities will be set.
        level : int
            Level for which N-Gram probabilities will be set. Probabilities for different levels of the N-gram can be set
            with this function. If level = 1, N-Gram is treated as UniGram, if level = 2, N-Gram is treated as Bigram, etc.
        """
        nGram.replaceUnknownWords(self.__dictionary)
        super().setProbabilities(nGram, level)
